/**
 * ProzessVerhaeltnis.java
 * eu.gronos.kostenrechner.calculation (Kostenrechner)
 */
package eu.gronos.kostenrechner.model.tenordaten;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;

/**
 * Klasse zum strukturierten Darstellen von Verurteilungen als Verhaeltnis
 * zwischen Gläubiger und Schuldner
 *
 * @author Peter Schuster (setrok)
 * @version
 * @date 04.07.2018
 *
 */
public abstract class ProzessVerhaeltnis implements Comparable<ProzessVerhaeltnis> {
	private Beteiligter glaeubiger;
	private Beteiligter schuldner;
	private boolean darueberHinaus;

	/**
	 * Konstruktor ohne Parameter für JAXB
	 * 
	 */
	public ProzessVerhaeltnis() {
	}

	/**
	 * Konstruktor mit allen Feldern zum raschen initialisieren
	 * 
	 * @param glaeubiger
	 * @param schuldner
	 * @param darueberHinaus
	 */
	public ProzessVerhaeltnis(Beteiligter glaeubiger, Beteiligter schuldner, boolean darueberHinaus) {
		this();
		this.glaeubiger = glaeubiger;
		this.schuldner = schuldner;
		this.darueberHinaus = darueberHinaus;
	}

	/**
	 * @return gibt {@link #glaeubiger} als Beteiligter zurück.
	 */
	@XmlElement(name = "glaeubiger")
	public Beteiligter getGlaeubiger() {
		return glaeubiger;
	}

	/**
	 * @param glaeubiger d. {@link #glaeubiger}, d. gesetzt werden soll als
	 *                   Beteiligter
	 */
	public void setGlaeubiger(Beteiligter glaeubiger) {
		this.glaeubiger = glaeubiger;
	}

	/**
	 * @return gibt {@link #schuldner} als Beteiligter zurück.
	 */
	@XmlElement(name = "schuldner")
	public Beteiligter getSchuldner() {
		return schuldner;
	}

	/**
	 * @param schuldner d. {@link #schuldner}, d. gesetzt werden soll als
	 *                  Beteiligter
	 */
	public void setSchuldner(Beteiligter schuldner) {
		this.schuldner = schuldner;
	}

	/**
	 * @return gibt {@link #darueberHinaus} als boolean zurück.
	 */
	@XmlAttribute(name = "darueberHinaus")
	public boolean isDarueberHinaus() {
		return darueberHinaus;
	}

	/**
	 * @param darueberHinaus d. {@link #darueberHinaus}, d. gesetzt werden soll als
	 *                       boolean
	 */
	public void setDarueberHinaus(boolean darueberHinaus) {
		this.darueberHinaus = darueberHinaus;
	}

	/**
	 * Die Methode dient dazu, zwei Prozessverhältnisse zu vergleichen, um sie
	 * sortieren zu können. Stellt zunächst auf die Gläubiger, dann auf die
	 * Schuldner ab.
	 * 
	 * @param o
	 * @return
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(ProzessVerhaeltnis o) {
		if (equals(o))
			return 0;
		if (getGlaeubiger().equals(o.getGlaeubiger()))
			return getSchuldner().compareTo(o.getSchuldner());
		else
			return getGlaeubiger().compareTo(o.getGlaeubiger());
	}

	/**
	 * Die Methode dient zur Prüfung, ob zwei ProzessVerhaeltnisse gleich sind, also
	 * die gleiche Klasse, den gleichen Gläubiger und den gleichen Schuldner haben.
	 * 
	 * @param obj das andere Prozessverhältnis
	 * @return
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		if (obj == this)
			return true;
		if (getClass() != obj.getClass())
			return false;
		ProzessVerhaeltnis anderesPV = (ProzessVerhaeltnis) obj;
		return getGlaeubiger().equals(anderesPV.getGlaeubiger()) && getSchuldner().equals(anderesPV.getSchuldner());
	}

}
