/**
 * KostenTenorLadenSpeichern.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.view;

import java.awt.Component;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.xml.bind.JAXBException;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.controller.files.BerechnungXmlDatei;
import eu.gronos.kostenrechner.controller.files.VerfahrensDatenContainerUtil;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Eine Klasse, die alle Zustände des Programms im XML-Format speichern kann.
 *
 * @author Peter Schuster (setrok)
 * @date 26.12.2014
 *
 * @url "http://java.kompf.de/jaxbjava.html"
 * @url "http://docs.oracle.com/javase/tutorial/jaxb/intro/"
 */
public class BerechnungLadenSpeichernDialog extends JFileChooser {

	private static final long serialVersionUID = 5816976530887334541L;
	static final FileNameExtensionFilter filter = new FileNameExtensionFilter(VerfahrensDatenContainerUtil.DATEITYP,
			VerfahrensDatenContainerUtil.DATEIENDUNG);
	public BerechnungXmlDatei berechnungXml;

	/**
	 * Konstruktor ohne Parameter
	 * 
	 */
	public BerechnungLadenSpeichernDialog() {
		super();
		setFileFilter(filter);
		berechnungXml = new BerechnungXmlDatei();
	}

	/**
	 * Die Methode dient dazu, einen Dateiauswahldialog der {@link JFileChooser
	 * Oberklasse} aufzurufen um dann mit {@link #MISSING() speicherXml} die
	 * übergebenen Programmzustände als XML zu speichern.
	 * 
	 * @param parent    die aufrufende {@link Component}
	 * @param container der {@link VerfahrensDatenContainer} mit allen zu
	 *                  speichernden Zuständen
	 * @return den Rückgabestatus des {@link JFileChooser}s:<br>
	 *         <ul>
	 *         <li>{@link JFileChooser#CANCEL_OPTION}</li>
	 *         <li>{@link JFileChooser#APPROVE_OPTION}</li>
	 *         <li>{@link JFileChooser#ERROR_OPTION}, wenn der Dialog einen Fehler
	 *         auslöst, er abgebrochen wird oder es einen Fehler beim Laden
	 *         gibt.</li>
	 * 
	 * @see JFileChooser#showSaveDialog(Component)
	 * @see JFileChooser#showDialog(Component, String)
	 */
	public int speicherXmlDialog(Component parent, VerfahrensDatenContainer container) {
		int option = super.showSaveDialog(parent);
		if (option == JFileChooser.APPROVE_OPTION)
			try {
				File datei = super.getSelectedFile();
				if (!datei.getName().endsWith(VerfahrensDatenContainerUtil.DATEIENDUNG_GANZ))
					datei = new File(datei.getAbsolutePath() + VerfahrensDatenContainerUtil.DATEIENDUNG_GANZ);
				container.allgemein.speicherdatum = VerfahrensDatenContainerUtil.speicherDatum();
				berechnungXml.speicherXml(container, datei);
				Kostenrechner.getLogger().info(String.format("Gespeichert: %s", datei.getAbsolutePath()));
				// System.out.println("Gespeichert: " + datei.getAbsolutePath());
			} catch (JAXBException je) {
				option = JFileChooser.ERROR_OPTION;
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, "Fehler beim Speichern als XML!", je);
			}
		return option;
	}

	/**
	 * Die Methode dient dazu, einen Dateiauswahldialog der {@link JFileChooser
	 * Oberklasse} aufzurufen um dann mit {@link #MISSING()} aus der ausgewählte
	 * Datei einen {@link VerfahrensDatenContainer} auszulesen.
	 * 
	 * @param parent die aufrufende {@link Component}
	 * @return einen {@link VerfahrensDatenContainer} zum Auslesen
	 * 
	 * @see JFileChooser#showOpenDialog(Component)
	 * @see JFileChooser#showDialog(Component, String)
	 * @see #MISSING()
	 */
	public VerfahrensDatenContainer ladeXmlDialog(Component parent) {
		VerfahrensDatenContainer container = null;
		int option = super.showOpenDialog(parent);
		if (option == JFileChooser.APPROVE_OPTION) {
			File datei = super.getSelectedFile();
			container = berechnungXml.ladeXml(datei);
		}
		return container;
	}

	// private static KostenTenorContainer erstelleContainer() {
	// KostenTenorContainer container = new KostenTenorContainer();
	// Beteiligter kl = new Beteiligter(Beteiligter.KLAEGER,
	// Beteiligter.MAENNLICH);
	// Beteiligter bk = new Beteiligter(Beteiligter.BEKLAGTE,
	// Beteiligter.WEIBLICH);
	//
	// /*
	// * Allgemeine Programmeinstellungen, die nicht vom Container selbst
	// * gesetzt werden
	// */
	// container.allgemein.b92ii = true;
	//
	// /* Erste Registerkarte Teilklagerücknahme */
	// container.teilklageruecknahme.parteien.add(kl);
	// container.teilklageruecknahme.parteien.add(bk);
	// container.teilklageruecknahme.streitwerteUndObsiegen.add(10000L);
	// container.teilklageruecknahme.streitwerteUndObsiegen.add(5000L);
	// container.teilklageruecknahme.streitwerteUndObsiegen.add(5000L);
	// container.teilklageruecknahme.vorher = new GebuehrenTatbestand[] {
	// GerichtsGebuehrenTabelle.KV1210,
	// AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3100, };
	// container.teilklageruecknahme.nachher = new GebuehrenTatbestand[] {
	// AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV3104, };
	//
	// /* Zweite Registerkarte Streitgenossen */
	// container.streitgenossen.widerklage = false;
	// container.streitgenossen.wirtschaftlichIdentisch = false;
	// container.streitgenossen.klaeger = new BaumbachBeteiligtenListe();
	// container.streitgenossen.klaeger.add(new BaumbachBeteiligter(
	// Beteiligter.KLAEGER, Beteiligter.MAENNLICH, 0.0, 0.0, false));
	// container.streitgenossen.drittwiderbeklagte = new
	// BaumbachBeteiligtenListe();
	// container.streitgenossen.beklagte = new BaumbachBeteiligtenListe();
	// container.streitgenossen.beklagte.add(new BaumbachBeteiligter(
	// Beteiligter.BEKLAGTE, Beteiligter.WEIBLICH, 1200, 1200, false));
	// container.streitgenossen.beklagte.add(new BaumbachBeteiligter(
	// Beteiligter.BEKLAGTE, Beteiligter.WEIBLICH, 1200, 0, false));
	// container.streitgenossen.beklagte
	// .add(new BaumbachGesamtschuldnerschaft(
	// Beteiligter.BEKLAGTE,
	// Beteiligter.MAENNLICH,
	// 0,
	// false,
	// new int[] { 0, 1 },
	// (BaumbachBeteiligtenListe) container.streitgenossen.beklagte));
	//
	// /* Dritte Registerkarte Forderungsstaffelung */
	// container.staffelung.parteien.add(kl);
	// container.staffelung.parteien.add(bk);
	// container.staffelung.forderungen
	// .add(new KlageForderung(10000.0, 7500.0));
	// container.staffelung.forderungen.add(new AufrechnungsForderung(7500.0,
	// 5000.0));
	// container.staffelung.forderungen.add(new AufrechnungsForderung(2500.0,
	// 1000.0));
	//
	// /* Vierte Registerkarte Freie Gebührenberechnung */
	// container.gebuehrenBerechnung.streitwert = 10000L;
	// container.gebuehrenBerechnung.gebuehren = new GebuehrenTatbestand[] {
	// GerichtsGebuehrenTabelle.KV1210,
	// AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV1008,
	// AnwaltsGebuehrenTabelle.VV3104,
	// GerichtsGebuehrenTabelle.KV9003,
	// AnwaltsGebuehrenTabelle.VV7008_R,
	// GerichtsGebuehrenTabelle.KV9003 };
	//
	// return container;
	// }

	// public static void main(String[] arg) throws JAXBException {
	// KostenTenorXmlLadenSpeichern db = new KostenTenorXmlLadenSpeichern();
	// KostenTenorContainer container = erstelleContainer();
	// db.speicherXmlDialog(null, container);
	// db.ladeXmlDialog(null);
	// }
}
