/**
 * KostenJPanel.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view;

import java.awt.GridBagLayout;
import java.awt.KeyboardFocusManager;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.text.NumberFormat;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.text.NumberFormatter;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenTransferHandler;
import eu.gronos.kostenrechner.interfaces.RowHandler;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;
import eu.gronos.kostenrechner.view.forderungen.ForderungsStaffelungPanel;
import eu.gronos.kostenrechner.view.gebuehren.GebuehrenBerechnungPanel;
import eu.gronos.kostenrechner.view.gebuehren.GebuehrenRenderer;
import eu.gronos.kostenrechner.view.gebuehren.TeilklageruecknahmePanel;
import eu.gronos.kostenrechner.view.gebuehren.TooltipHeaderRenderer;

/**
 * Abstrakte Oberklasse, von denen die Klassen für die Registerkarten abgeleitet werden können.
 *
 * @author Peter Schuster (setrok)
 * @version
 * @date 13.06.2018
 *
 */
public abstract class KostenJPanel extends JPanel {

	private static final long serialVersionUID = -6667909235024117172L;
	/**
	 * Der AbstractAction kann JPopupMenus hinzugefügt werden. Soll für die
	 * Beteiligtentabellen und die Gebührentabellen einsetzbar sein.
	 * {@link KeyEvent#VK_E}
	 */
	public final AbstractAction eintragEntfernenAktion = new BeschriebeneAktion("Eintrag entfernen",
			"Entfernt den markierten Eintrag aus der Tabelle", KeyEvent.VK_E, "remove") {
		private static final long serialVersionUID = 8001075205760187469L;
	
		@Override
		public void actionPerformed(ActionEvent ae) {
			Object o = ae.getSource();
			if (!(o instanceof JMenuItem)) {
				return;
			}
			// Herausfinden, welche Tabelle ruft
			JPopupMenu popup = (JPopupMenu) ((JMenuItem) o).getParent();
			if (popup.getLabel() == TeilklageruecknahmePanel.VORHER_POPUPLABEL)
				entferneTabellenEintrag(TeilklageruecknahmePanel.getInstance().tabVorher, TeilklageruecknahmePanel.getInstance().getGttmVorher());
			if (popup.getLabel() == TeilklageruecknahmePanel.NACHHER_POPUPLABEL)
				entferneTabellenEintrag(TeilklageruecknahmePanel.getInstance().tabNachher, TeilklageruecknahmePanel.getInstance().getGttmNachher());
			if (popup.getLabel() == StreitgenossenPanel.BEKLAGTE_POPUPLABEL)
				StreitgenossenPanel.getInstance().entferneBeteiligtenEintrag(StreitgenossenPanel.getInstance().tabBeklagte, StreitgenossenPanel.getInstance().btmBeklagte);
			if (popup.getLabel() == StreitgenossenPanel.KLAEGER_POPUPLABEL)
				StreitgenossenPanel.getInstance().entferneBeteiligtenEintrag(StreitgenossenPanel.getInstance().tabKlaeger, StreitgenossenPanel.getInstance().btmKlaeger);
			if (popup.getLabel() == StreitgenossenPanel.DRITTWIDERBEKLAGTE_POPUPLABEL)
				StreitgenossenPanel.getInstance().entferneBeteiligtenEintrag(StreitgenossenPanel.getInstance().tabDrittwiderbeklagte, StreitgenossenPanel.getInstance().btmDrittwiderbeklagte);
			if (popup.getLabel() == ForderungsStaffelungPanel.FORDERUNGEN_POPUPLABEL)
				entferneTabellenEintrag(ForderungsStaffelungPanel.getInstance().tbForderungen, ForderungsStaffelungPanel.getInstance().ftm);
			if (popup.getLabel() == GebuehrenBerechnungPanel.RECHNER_POPUPLABEL)
				entferneTabellenEintrag(GebuehrenBerechnungPanel.getInstance().tabGebuehrenTatbestaende, GebuehrenBerechnungPanel.getInstance().gttmRechner);
		}
	};
	/**
	 * tabNext: AbstractAction, die verhindert, dass die JTable die
	 * Tabulartortaste "frisst".
	 * 
	 * 
	 * @url "http://www.coderanch.com/t/342895/GUI/java/JTable-focus" * aus: Big
	 *      Moose Saloon, A friendly place for programming greenhorns!
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	AbstractAction aaTabNext = new AbstractAction() {
		private static final long serialVersionUID = -6095777609039741786L;

		public void actionPerformed(ActionEvent ae) {
			KeyboardFocusManager.getCurrentKeyboardFocusManager().focusNextComponent();
		}
	};

	/**
	 * AbstractAction, die verhindert, dass die JTable Umsch+Tabulartortaste
	 * "frisst".
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	AbstractAction aaTabBefore = new AbstractAction() {
		private static final long serialVersionUID = -244188598744791079L;

		public void actionPerformed(ActionEvent ae) {
			KeyboardFocusManager.getCurrentKeyboardFocusManager().focusPreviousComponent();
		}
	};
	public static final NumberFormatter FORMATTER_INT = new NumberFormatter(NumberFormat.getIntegerInstance());

	/**
	 * Dank an JavaBeginners, Mit einem NumberFormatter lässt sich ein
	 * TextFeld so formatieren, dass nur Zahlen als Eingabe akzeptiert
	 * werden. Die Grundlage hierfür bildet ein JFormattedTextField.
	 * 
	 * @url
	 * "http://www.javabeginners.de/Swing/Swing-Elemente/Textkomponenten/Textfeld_fuer_Zahlen.php"
	 */
	public static final NumberFormat FORMAT = NumberFormat.getInstance();
	public static final NumberFormatter FORMATTER = new NumberFormatter(FORMAT);
	public static final TooltipHeaderRenderer HEADER_RENDERER = new TooltipHeaderRenderer();
	public static final EuroEditor EURO_EDITOR = new EuroEditor(FORMATTER);
	public static final EuroRenderer EURO_RENDERER = new EuroRenderer();

	/**
	 * Konstruktor setzt das GridBagLayout als LayoutManager
	 * 
	 */
	protected KostenJPanel() {
		//super(new GridBagLayout());
		super(new GridBagLayout());
		FORMAT.setGroupingUsed(false);
	}

	/**
	 * Die Methode dient dazu, die Oberflächenelemente für das Panel zu setzen, wie bisher fuelleXxxPane()
	 * 
	 */
	public abstract void buildPanel();

	/**
	 * Die Methode dient dazu, eine Referenz auf das Hauptfenster zu übermitteln
	 * 
	 * @param kostenrechner
	 */
	public abstract void setKostenrechner(Kostenrechner kostenrechner) ;
	


	/**
	 * Verhindert, dass die übergegebene <code>JTable table</code> die
	 * Tabulatortaste und Umschalt+Tabulatortaste "frisst".
	 * 
	 * @param table
	 *            die JTable
	 * @url "http://www.coderanch.com/t/342895/GUI/java/JTable-focus" aus: Big
	 *      Moose Saloon, A friendly place for programming greenhorns!
	 */
	public void entferneTastaturFalle(JTable table) {
		table.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0), "tabNext");
		table.getActionMap().put("tabNext", aaTabNext);
		table.getInputMap(JComponent.WHEN_FOCUSED)
				.put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.SHIFT_DOWN_MASK), "tabBefore");
		table.getActionMap().put("tabBefore", aaTabBefore);
	}

	/**
	 * Entfernt eine Zeile aus einer Gebühren- oder Forderungs-Tabelle.
	 * 
	 * @param table
	 *            die JTable, aus der die Zeile gelöscht werden soll
	 * @param model
	 *            das RowHandler-implementierende TableModel, aus der die Zeile
	 *            gelöscht werden soll
	 */
	protected void entferneTabellenEintrag(JTable table, RowHandler<?> model) {
		if (table == null || model == null || table.getSelectedRow() < 0)
			return;
		int selectedRow = table.getSelectedRow();
		model.removeRow(selectedRow);
	}

	/**
	 * Die Methode dient dazu, bei den JTables für die GebührenTatbestände die
	 * Spaltenbreite einzustellen, den Renderer und den Transferhandler zu
	 * setzen.
	 * 
	 * @param table
	 *            eine JTable für GebührenTatbestände
	 * 
	 */
	public void setzeGebuehrenTabelleAussehen(JTable table) {
		table.getColumnModel().getColumn(0).setMinWidth(180);
		table.getColumnModel().getColumn(0).setPreferredWidth(180);
		table.getColumnModel().getColumn(1).setPreferredWidth(40);
		table.getColumnModel().getColumn(2).setPreferredWidth(60);
		table.setDefaultRenderer(Double.class, new GebuehrenRenderer());
		table.getTableHeader().setDefaultRenderer(HEADER_RENDERER);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setTransferHandler(new GebuehrenTransferHandler(table));
	}

}