/**
 * GesamtschuldnerDialog.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.baumbach;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.event.KeyEvent;

import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligtenListe;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligter;
import eu.gronos.kostenrechner.model.baumbach.BaumbachGesamtschuldnerschaft;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Dialog zum Hinzufügen eines BaumbachGesamtschuldners
 * 
 * @author Peter Schuster (setrok)
 * @date 16.06.2014
 */
public // class GesamtschuldnerDialog extends JDialog {
class GesamtschuldnerDialog extends
		HinzufuegenDialog<BaumbachGesamtschuldnerschaft> {
	/**
	 * Der long mit dem Namen serialVersionUID speichert...
	 */
	private static final long serialVersionUID = 7906671533792996231L;

	/**
	 * Der EmptyBorder mit dem Namen BORDER speichert...
	 */
	// private static final EmptyBorder BORDER = new EmptyBorder(5, 5, 5, 5);

	private BaumbachBeteiligtenListe baumbachBeteiligtenListe;

	// private JPanel contentPane;

	// private NumberFormat format;

	// private NumberFormatter formatter;

	// protected boolean isOk;

	private JScrollPane scpBeteiligtenAuswahlListe;

	private JList<BaumbachBeteiligter> beteiligtenAuswahlListe;

	private JFormattedTextField ftfUnterliegen;

	/**
	 * Erzeugt einen modalen Dialog zum Hinzufügen eines Gesamtschuldners
	 * 
	 * @param owner
	 *            the JFrame from which the dialog is displayed
	 * @param kl
	 *            die BaumbachBeteiligtenListe mit den Klägern
	 * @param dwbk
	 *            die BaumbachBeteiligtenListe mit den Drittwiderbeklagten, zu
	 *            der ein Beteiligter hinzugefügt werden soll
	 */
	public GesamtschuldnerDialog(JFrame owner, BaumbachBeteiligtenListe kl,
			BaumbachBeteiligtenListe dwbk) {
		this(owner, new BaumbachBeteiligtenListe(kl, dwbk));
	}

	/**
	 * Erzeugt einen modalen Dialog zum Hinzufügen eines Gesamtschuldners
	 * 
	 * @param owner
	 *            the JFrame from which the dialog is displayed
	 * @param baumbachBeteiligtenListe
	 *            die BaumbachBeteiligtenListe, zu der ein Beteiligter
	 *            hinzugefügt werden soll
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#HinzufuegenDialog(JFrame,String)
	 * @see javax.swing.JDialog#JDialog(Frame, String, boolean)
	 * @throws IllegalArgumentException
	 *             wenn die <code>baumbachBeteiligtenListe</code> leer ist
	 */
	public GesamtschuldnerDialog(JFrame owner,
			BaumbachBeteiligtenListe baumbachBeteiligtenListe)
			throws IllegalArgumentException {
		// super(owner, "Gesamtschuldner hinzufügen", true);
		super(owner, "Gesamtschuldnerische Verurteilung hinzufügen");
		if (baumbachBeteiligtenListe.size() < 1)
			throw new IllegalArgumentException(
					"Keine Beteiligten für den angegebenen Beteiligtentyp vorhanden, "
							+ "\ndie gesamtschuldnerisch verurteilt werden könnten. "
							+ "\nBitte zunächst \"einfache\" Beteiligte der Liste hinzufügen.");
		else
			this.baumbachBeteiligtenListe = baumbachBeteiligtenListe;
	}

	/**
	 * Die Methode greift den letzten Eintrag aus der übergebenen
	 * baumbachBeteiligtenListe
	 * 
	 * @return
	 */
	private BaumbachBeteiligter getLetztenBeteiligten() {
		return this.baumbachBeteiligtenListe.get(this.baumbachBeteiligtenListe
				.size() - 1);
	}

	/**
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#baueRueckgabewert()
	 */
	@Override
	protected BaumbachGesamtschuldnerschaft baueRueckgabewert() {
		int typ = getLetztenBeteiligten().getTyp();
		if (typ == Beteiligter.KLAEGER)
			typ = Beteiligter.DRITTWIDERBEKLAGTE;
		int[] indices = beteiligtenAuswahlListe.getSelectedIndices();
		int genusNumerus = getLetztenBeteiligten().getGenusNumerus();
		if (genusNumerus < Beteiligter.PLURAL)
			genusNumerus += Beteiligter.PLURAL;
		// anWiderklageBeteiligter darf beim gesamtschuldnerischen Beklagten
		// nicht gesetzt werden (eine Widerklage führt man nicht
		// gesamtschuldnerisch)
		final boolean anWiderklageBeteiligt = baumbachBeteiligtenListe
				.enthaeltAnWiderklageBeteiligte()
				&& typ != Beteiligter.BEKLAGTE;
		double unterliegen = 0.0;
		unterliegen = Double.parseDouble(ftfUnterliegen.getText());
		return new BaumbachGesamtschuldnerschaft(typ, genusNumerus,
				unterliegen, anWiderklageBeteiligt, indices,
				baumbachBeteiligtenListe);
	}

	/**
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#fuelleContentPane()
	 */
	@Override
	protected void fuelleContentPane() {
		{
			JLabel lblBeteiligtentyp = new JLabel(
					"Gesamtschuldnerisch verurteilte Beteiligte auswählen");
			contentPanel.add(lblBeteiligtentyp, GitterBeutelBeschraenkungen
					.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL,
							false));
			beteiligtenAuswahlListe = new JList<BaumbachBeteiligter>(
					baumbachBeteiligtenListe
							.toArray(new BaumbachBeteiligter[baumbachBeteiligtenListe
									.size()]));
			beteiligtenAuswahlListe
					.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
			beteiligtenAuswahlListe
					.setCellRenderer(StreitgenossenPanel.BETEILIGTEN_RENDERER);
			scpBeteiligtenAuswahlListe = new JScrollPane(
					beteiligtenAuswahlListe);
			setMnemonicLabelFor(lblBeteiligtentyp, scpBeteiligtenAuswahlListe,
					KeyEvent.VK_B);
			contentPanel.add(scpBeteiligtenAuswahlListe,
					GitterBeutelBeschraenkungen.getInstance(0, 1, 2, 1,
							GridBagConstraints.BOTH, true));
		}
		{
			JLabel lblUnterliegen = new JLabel("Höhe der Verurteilung");
			contentPanel.add(lblUnterliegen, GitterBeutelBeschraenkungen
					.getInstance(0, 2, 1, 1, GridBagConstraints.HORIZONTAL,
							false));
			ftfUnterliegen = new JFormattedTextField(formatter);
			ftfUnterliegen.setText(Double.toString(getLetztenBeteiligten()
					.getUnterliegen()));
			ftfUnterliegen.setColumns(10);
			setEnterAction(ftfUnterliegen, getOkAction());
			setMnemonicLabelFor(lblUnterliegen, ftfUnterliegen, KeyEvent.VK_V);
			contentPanel.add(ftfUnterliegen, GitterBeutelBeschraenkungen
					.getInstance(1, 2, 1, 1, GridBagConstraints.HORIZONTAL,
							false));
		}
	}

	/**
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#parseEingabe()
	 */
	@Override
	protected boolean parseEingabe() {
		try {
			Double.parseDouble(ftfUnterliegen.getText());
			int[] indices = beteiligtenAuswahlListe.getSelectedIndices();
			if (indices == null || indices.length < 2)
				throw new NullPointerException(
						"Nicht genug Beteiligte ausgewählt. Wählen Sie mindestens zwei Beteiligte aus!");
			// TODO Gesamtschuldnerschaften dürfen nicht ausgewählt sein
		} catch (NumberFormatException | NullPointerException e) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
			e.printStackTrace();
			return false;
		}
		return true;
	}
}
