/**
 * AufrechnungPanel.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.forderungen;

import java.awt.GridBagConstraints;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.KontextMouseListener;
import eu.gronos.kostenrechner.controller.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.controller.VerfahrensDatenContainer.ForderungsStaffelung;
import eu.gronos.kostenrechner.controller.forderungen.ForderungNeuAction;
import eu.gronos.kostenrechner.controller.forderungen.ForderungsStaffelungTenorAction;
import eu.gronos.kostenrechner.interfaces.WerteLesbar;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligter;
import eu.gronos.kostenrechner.model.forderungen.Forderung;
import eu.gronos.kostenrechner.model.forderungen.ForderungenTableModel;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.KostenJPanel;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;

/**
 * Diese von {@link KostenJPanel} abgeleitete Klasse nimmt die
 * Oberflächenelemente für die Forderungsstaffelung auf.
 *
 * @author Peter Schuster (setrok)
 * @version
 * @date 13.06.2018
 *
 */
public class ForderungsStaffelungPanel extends KostenJPanel
		implements WerteLesbar<VerfahrensDatenContainer.ForderungsStaffelung> {

	private static final long serialVersionUID = 1506079463701863627L;
	private static final ForderungsStaffelungPanel instance = new ForderungsStaffelungPanel();
	private Kostenrechner kostenrechner;
	public JComboBox<Beteiligter> cbAngriffsKlaeger;
	public AbstractAction angriffsVerteidigungsmittelStaffelungAction = new ForderungsStaffelungTenorAction(this, "Tenor ermitteln", "Ermittelt den Kostentenor und Streitwert bei Staffelung von Hauptsache und Aufrechnungen.", KeyEvent.VK_T, "aufrechnung-tenor");
	public JComboBox<Beteiligter> cbVerteidigungsBeklagter;
	public ForderungenTableModel ftm;
	public AbstractAction neueForderungAction = new ForderungNeuAction(this, "Angriffs-/Verteidigungsmittel hinzufügen", "Fügt ein neues Angriffs- oder Verteidigungsmittel zur Tabelle hinzu", KeyEvent.VK_H, "forderung-neu");
	public JTable tbForderungen;
	public JPopupMenu pmForderungen;
	private static final ForderungenRenderer FORDERUNGEN_RENDERER = new ForderungenRenderer();
	public static final String FORDERUNGEN_POPUPLABEL = "Angriffs-/Verteidigungsmittel...";

	/**
	 * Der Konstruktor ruft nur die Oberklasse auf.
	 * 
	 */
	private ForderungsStaffelungPanel() {
		super();
	}

	/**
	 * @return gibt {@link #instance} als AufrechnungPanel zurück.
	 */
	public static ForderungsStaffelungPanel getInstance() {
		return instance;
	}

	/**
	 * Die Methode setzt die Oberflächenelemente für die Registerkarte 3
	 * Hilfsaufrechnung.
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#buildPanel()
	 */
	@Override
	public void buildPanel() {
		JLabel lblParteien = new JLabel("Parteien:", SwingConstants.RIGHT);
		lblParteien.setDisplayedMnemonic(KeyEvent.VK_P);
		add(lblParteien, GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		Beteiligter[] klBezeichnerListe = BaumbachBeteiligter.getBeteiligtenAuswahlListe(Beteiligter.KLAEGER, true);
		cbAngriffsKlaeger = new JComboBox<Beteiligter>(klBezeichnerListe);
		cbAngriffsKlaeger.setRenderer(StreitgenossenPanel.BETEILIGTEN_RENDERER_OHNE_LFDNR);
		lblParteien.setLabelFor(cbAngriffsKlaeger);
		add(cbAngriffsKlaeger,
				GitterBeutelBeschraenkungen.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		Beteiligter[] bkBezeichnerListe = BaumbachBeteiligter.getBeteiligtenAuswahlListe(Beteiligter.BEKLAGTE, true);
		cbVerteidigungsBeklagter = new JComboBox<Beteiligter>(bkBezeichnerListe);
		cbVerteidigungsBeklagter.setRenderer(StreitgenossenPanel.BETEILIGTEN_RENDERER_OHNE_LFDNR);
		add(cbVerteidigungsBeklagter,
				GitterBeutelBeschraenkungen.getInstance(1, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblForderungen = new JLabel("Bitte Angriffs- und Verteidigungsmittel angeben:");
		lblParteien.setDisplayedMnemonic(KeyEvent.VK_V);
		add(lblForderungen, GitterBeutelBeschraenkungen.getInstance(0, 2, 1, 1, GridBagConstraints.HORIZONTAL, false));

		tbForderungen = new JTable();
		// kostenrechner.setzeForderungsTabellen();
		leereWerte();
		entferneTastaturFalle(tbForderungen);
		// tbForderungen.setFillsViewportHeight(true);
		tbForderungen.setDefaultRenderer(Forderung.class, ForderungsStaffelungPanel.FORDERUNGEN_RENDERER);
		tbForderungen.setDefaultRenderer(Double.class, KostenJPanel.EURO_RENDERER);
		tbForderungen.setDefaultEditor(Double.class, KostenJPanel.EURO_EDITOR);
		JComboBox<Forderung> cbForderungen = new JComboBox<Forderung>(ForderungHinzufuegen.sammleForderungen());
		cbForderungen.setRenderer(ForderungsStaffelungPanel.FORDERUNGEN_RENDERER);
		tbForderungen.setDefaultEditor(Forderung.class, new DefaultCellEditor(cbForderungen));
		tbForderungen.getTableHeader().setDefaultRenderer(KostenJPanel.HEADER_RENDERER);

		JScrollPane scpForderungen = new JScrollPane(tbForderungen);
		lblForderungen.setLabelFor(scpForderungen);
		add(scpForderungen, GitterBeutelBeschraenkungen.getInstance(0, 3, 2, 1, GridBagConstraints.BOTH, false));

		JButton btnNeueForderung = new JButton(neueForderungAction);
		add(btnNeueForderung,
				GitterBeutelBeschraenkungen.getInstance(0, 4, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JButton btnAngriffsVerteidigungsmittelStaffelung = new JButton(angriffsVerteidigungsmittelStaffelungAction);
		add(btnAngriffsVerteidigungsmittelStaffelung,
				GitterBeutelBeschraenkungen.getInstance(0, 5, 2, 1, GridBagConstraints.HORIZONTAL, false));

		pmForderungen = new JPopupMenu(ForderungsStaffelungPanel.FORDERUNGEN_POPUPLABEL);
		pmForderungen.add(new JMenuItem(ForderungsStaffelungPanel.FORDERUNGEN_POPUPLABEL));
		pmForderungen.addSeparator();
		pmForderungen.add(new JMenuItem(neueForderungAction));
		pmForderungen.add(new JMenuItem(eintragEntfernenAktion));
		tbForderungen.addMouseListener(new KontextMouseListener(tbForderungen, pmForderungen));
	}

	/**
	 * Die Methode leert die Tabelle mit den Forderungen und ersetzt mit denen aus
	 * dem {@link ForderungsStaffelung}-Behälter. Zudem werden
	 * {@link #cbAngriffsKlaeger} und {@link #cbVerteidigungsBeklagter} angepasst.
	 * 
	 * @param staffelung {@link VerfahrensDatenContainer.ForderungsStaffelung}
	 *
	 * @see eu.gronos.kostenrechner.interfaces.WerteLesbar#setzeWerte(java.lang.Object)
	 */
	@Override
	public void setzeWerte(final ForderungsStaffelung container) {
		cbAngriffsKlaeger.setSelectedItem(container.parteien.get(0));
		cbVerteidigungsBeklagter.setSelectedItem(container.parteien.get(1));
		ftm = new ForderungenTableModel();
		ftm.setAllValues(container.forderungen);
		tbForderungen.setModel(ftm);
	}

	/**
	 * Die Methode leert die Tabelle mit den Forderungen.
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.WerteLesbar#leereWerte()
	 */
	public void leereWerte() {
		if (tbForderungen != null) {
			ftm = new ForderungenTableModel();
			tbForderungen.setModel(ftm);
		}
	}

	/**
	 * Die Methode dient dazu, alle Programmzustände in einen
	 * {@link VerfahrensDatenContainer} zu packen
	 * 
	 * @return den Untercontainer von {@link VerfahrensDatenContainer#staffelung}
	 * @see eu.gronos.kostenrechner.interfaces.WerteLesbar#leseWerte()
	 */
	@Override
	public ForderungsStaffelung leseWerte() {
		ForderungsStaffelung container = new ForderungsStaffelung();
		container.parteien.add((Beteiligter) cbAngriffsKlaeger.getSelectedItem());
		container.parteien.add((Beteiligter) cbVerteidigungsBeklagter.getSelectedItem());
		container.forderungen = ftm.getAllValues();

		return container;
	}

	/**
	 * Die Methode dient dazu, eine Referenz auf das Hauptfenster zu übermitteln
	 * 
	 * @param kostenrechner
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#setKostenrechner(eu.gronos.kostenrechner.Kostenrechner)
	 */
	public void setKostenrechner(Kostenrechner kostenrechner) {
		this.kostenrechner = kostenrechner;
	}

	/**
	 * @return gibt {@link #kostenrechner} als Kostenrechner zurück.
	 */
	public Kostenrechner getKostenrechner() {
		return kostenrechner;
	}

}
