/**
 * GebuehrenPanel.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.gebuehren;

import java.awt.GridBagConstraints;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.AllzweckHorcher;
import eu.gronos.kostenrechner.controller.KontextMouseListener;
import eu.gronos.kostenrechner.controller.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.controller.VerfahrensDatenContainer.GebuehrenBerechnung;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrNeuAction;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenAufstellungAction;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenSetzeRechnerTabelleAction;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenStreitwertHorcher;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenTabelleDruckenAction;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenTabelleKopierenAction;
import eu.gronos.kostenrechner.interfaces.TableModelGebend;
import eu.gronos.kostenrechner.interfaces.WerteLesbar;
import eu.gronos.kostenrechner.model.gebuehren.AnwaltsGebuehrenTabelle;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestandTableModel;
import eu.gronos.kostenrechner.model.gebuehren.GerichtsGebuehrenTabelle;
import eu.gronos.kostenrechner.model.gebuehren.StreitwertTableModel;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.KostenJPanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Diese von {@link KostenJPanel} abgeleitete Klasse nimmt die
 * Oberflächenelemente für die Gebührenberechnung auf.
 *
 * @author Peter Schuster (setrok)
 * @date 13.06.2018
 *
 */
public class GebuehrenBerechnungPanel extends KostenJPanel
		implements WerteLesbar<VerfahrensDatenContainer.GebuehrenBerechnung>, TableModelGebend<GebuehrenTatbestandTableModel>  {
	private static final String SUMME_EUR = "Summe: %,.2f EUR";
	private static final String STREITWERTE_POPUPLABEL = "Streitwerte...";
	public static final int GEBUEHREN_ANERKENNTNIS = 101;
	public static final int GEBUEHREN_SAEUMNIS = 102;
	public static final int GEBUEHREN_STANDARD = 103;
	public static final int GEBUEHREN_2INSTANZ = 104;

	private static final long serialVersionUID = -8251865293845494552L;
	private static final GebuehrenBerechnungPanel instance = new GebuehrenBerechnungPanel();
	private Kostenrechner kostenrechner;
	private JTable tabStreitwertTabelle;
	private JFormattedTextField ftfStreitwert;
	public JTable tabGebuehrenTatbestaende;
	private JTextField tfGebuehrenSumme;
	private JPopupMenu pmStreitwert;
	private JPopupMenu pmGttmRechner;
	public GebuehrenTatbestandTableModel gttmRechner;
	// new GebuehrenRvgVerguetungAction
	private final AbstractAction neueRvgVerguetungAction = new /*GebuehrenNeueGebuehrAction*/GebuehrNeuAction(this,
			"Neue Vergütung nach RVG",
			"<html><b>Neue Gebühr</b><br>Fügt einen neuen Vergütungstatbestand nach RVG in die Tabelle für die Gebühren zur freien Berechnung ein.</html>",
			KeyEvent.VK_R, "gebuehr-rvg", TABLE_GEBUEHREN, AnwaltsGebuehrenTabelle.getAllGebuehrenTatbestaende());
	private final AbstractAction neueGkgGebuehrAction = new /*GebuehrenNeueGebuehrAction*/GebuehrNeuAction(this, "Neue Gebühr nach GKG",
			"<html><b>Neue Gebühr</b><br>Fügt einen neuen Gebührentatbestand nach GKG in die Tabelle für die Gebühren zur freien Berechnung ein.</html>",
			KeyEvent.VK_N, "gebuehr-gkg", TABLE_GEBUEHREN, GerichtsGebuehrenTabelle.getAllGebuehrenTatbestaende());
	private final AbstractAction gebuehrenAufstellungAction = new GebuehrenAufstellungAction(this,
			"Ausführliche Gebührenaufstellung erstellen",
			"Erstellt eine Aufstellung der Gebühren und ihrer Summe zu dem ausgewählten Streitwert.", KeyEvent.VK_E,
			"gebuehren-aufstellung");
	public final AbstractAction normaleGebuehrenAction = new GebuehrenSetzeRechnerTabelleAction(this,
			GEBUEHREN_STANDARD, "Gebühren im Normalfall",
			"Leert die Tabelle mit den Gebührentatbeständen des Rechners und \nfüllt sie mit der Konstellation eines normalen erstinstanzlichen Verfahrens",
			KeyEvent.VK_H, "normal-gebuehren");
	public final AbstractAction saeumnisGebuehrenAction = new GebuehrenSetzeRechnerTabelleAction(this,
			GEBUEHREN_SAEUMNIS, "Gebühren bei Säumnis",
			"Leert die Tabelle mit den Gebührentatbeständen des Rechners und \nfüllt sie mit der Konstellation Gebühren bei Säumnis",
			KeyEvent.VK_S, "saeumnis-gebuehren");// new GebuehrenSaeumnisAction(this,
	public final AbstractAction anerkenntnisGebuehrenAction = new GebuehrenSetzeRechnerTabelleAction(this,
			GEBUEHREN_ANERKENNTNIS, "Gebühren bei Anerkenntnis",
			"Leert die Tabelle mit den Gebührentatbeständen des Rechners und \nfüllt sie mit der Konstellation Gebühren bei Anerkenntnis",
			KeyEvent.VK_G, "anerkenntnis-gebuehren");// new GebuehrenAnerkenntnisAction(this,
	private final AbstractAction mitInstanzGebuehrenAction = new GebuehrenSetzeRechnerTabelleAction(this,
			GEBUEHREN_2INSTANZ, "Gebühren einschließlich Berufung",
			"Leert die Tabelle mit den Gebührentatbeständen des Rechners und \nfüllt sie mit der Konstellation eines normalen erst- und zweitinstanzlichen Verfahrens",
			KeyEvent.VK_B, "mit-instanz-gebuehren");// new GebuehrenInstanzAction(this,

	private final AllzweckHorcher aktualisierungsHorcher = new GebuehrenStreitwertHorcher(this);
	private final AbstractAction druckeSwTabelleAction = new GebuehrenTabelleDruckenAction(this, "Tabelle drucken",
			"Druckt den Inhalt der Streitwerttabelle", KeyEvent.VK_D, "print-table",
			KeyStroke.getKeyStroke(KeyEvent.VK_P, Kostenrechner.ctrlMask));
	private final AbstractAction kopiereSwTabelleAction = new GebuehrenTabelleKopierenAction(this,
			"Streitwerttabelle kopieren", "Kopiert den Inhalt der Streitwerttabelle in die Zwischenablage",
			KeyEvent.VK_K, "copy-table", KeyStroke.getKeyStroke(KeyEvent.VK_C, Kostenrechner.ctrlMask));
	public static final String RECHNER_POPUPLABEL = "Gebührentatbestände...";
	public static final String FEHLER_DRUCKEN = "Fehler beim Drucken";
	private static final int TABLE_GEBUEHREN = 0;

	/**
	 * Der Konstruktor ruft nur die Oberklasse auf.
	 * 
	 */
	private GebuehrenBerechnungPanel() {
		super();
	}

	/**
	 * Das Panel ist als Singelton ausgestatltet. Diese Methode gibt die alleinige
	 * Instanz zurück.
	 * 
	 * @return gibt {@link #instance} als GebuehrenBerechnungPanel zurück.
	 */
	public static GebuehrenBerechnungPanel getInstance() {
		return instance;
	}

	/**
	 * Die Methode fuellt den GebuehrenRechnerPanel.
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#buildPanel()
	 */
	@Override
	public void buildPanel() {
		// JFormattedTextField für den Streitwert, das sich ständig mit der
		// JTable synchronisiert
		JLabel lblStreitwert = new JLabel("Bitte geben Sie den Streitwert ein:", SwingConstants.RIGHT);
		setFtfStreitwert(new JFormattedTextField(KostenJPanel.FORMATTER));
		getFtfStreitwert().setColumns(10);
		getFtfStreitwert().setText("0");
		getFtfStreitwert().addFocusListener(HinzufuegenDialog.BEI_FOCUS_MARKIEREN);
		getFtfStreitwert().setHorizontalAlignment(JTextField.RIGHT);
		lblStreitwert.setDisplayedMnemonic(KeyEvent.VK_S);
		lblStreitwert.setDisplayedMnemonicIndex(20);
		lblStreitwert.setLabelFor(getFtfStreitwert());

		// JScrollpane mit Gebührentabelle, die sich mit dem ftf synchronisiert
		JLabel lblStreitwertTabelle = new JLabel("Oder wählen Sie einen Streitwert aus:");
		StreitwertTableModel swtm = new StreitwertTableModel();
		setTabStreitwertTabelle(new JTable(swtm));
		getTabStreitwertTabelle().setDefaultRenderer(Long.class, KostenJPanel.EURO_RENDERER);
		getTabStreitwertTabelle().setDefaultRenderer(Double.class, KostenJPanel.EURO_RENDERER);
		getTabStreitwertTabelle().getTableHeader().setDefaultRenderer(KostenJPanel.HEADER_RENDERER);
		getTabStreitwertTabelle().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		entferneTastaturFalle(getTabStreitwertTabelle());
		JScrollPane scpStreitwertTabelle = new JScrollPane(getTabStreitwertTabelle());
		lblStreitwertTabelle.setDisplayedMnemonic(KeyEvent.VK_W);
		lblStreitwertTabelle.setLabelFor(scpStreitwertTabelle);

		getFtfStreitwert().addFocusListener(aktualisierungsHorcher);
		getFtfStreitwert().addKeyListener(aktualisierungsHorcher);
		getTabStreitwertTabelle().addFocusListener(aktualisierungsHorcher);
		getTabStreitwertTabelle().addMouseListener(aktualisierungsHorcher);
		getTabStreitwertTabelle().addKeyListener(aktualisierungsHorcher);
		getTabStreitwertTabelle().getSelectionModel().addListSelectionListener(aktualisierungsHorcher);

		// JScrollpane mit JTable mit gttm + JLabel
		JLabel lblGebuehrenTatbestaende = new JLabel("Bitte geben Sie die angefallenen Gebühren an:");
		tabGebuehrenTatbestaende = new JTable();
		setzeGebuehrenRechnerTabelle(0);
		entferneTastaturFalle(tabGebuehrenTatbestaende);
		setzeGebuehrenTabelleAussehen(tabGebuehrenTatbestaende);
		JScrollPane scpGebuehrenTatbestaende = new JScrollPane(tabGebuehrenTatbestaende);
		lblGebuehrenTatbestaende.setDisplayedMnemonic(KeyEvent.VK_G);
		lblGebuehrenTatbestaende.setDisplayedMnemonicIndex(33);
		lblGebuehrenTatbestaende.setLabelFor(scpGebuehrenTatbestaende);

		// JButton Gebühr hinzufügen
		JButton btnNeueVerguetung = new JButton(neueRvgVerguetungAction);
		JButton btnNeueGebuehr = new JButton(neueGkgGebuehrAction);

		// JTextField Summe berechnen
		tfGebuehrenSumme = new JTextField();
		tfGebuehrenSumme.setEditable(false);
		tfGebuehrenSumme.setHorizontalAlignment(SwingConstants.RIGHT);
		JButton btnSummeBerechnen = new JButton(gebuehrenAufstellungAction);

		errechneGebuehrenSumme();

		// Der JPanel hinzufügen
		add(lblStreitwert, GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(getFtfStreitwert(),
				GitterBeutelBeschraenkungen.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(lblStreitwertTabelle,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(scpStreitwertTabelle, GitterBeutelBeschraenkungen.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, true));
		add(lblGebuehrenTatbestaende,
				GitterBeutelBeschraenkungen.getInstance(0, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(tfGebuehrenSumme,
				GitterBeutelBeschraenkungen.getInstance(1, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(scpGebuehrenTatbestaende,
				GitterBeutelBeschraenkungen.getInstance(0, 4, 2, 1, GridBagConstraints.BOTH, true));
		add(btnNeueGebuehr, GitterBeutelBeschraenkungen.getInstance(0, 5, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(btnNeueVerguetung,
				GitterBeutelBeschraenkungen.getInstance(1, 5, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(btnSummeBerechnen,
				GitterBeutelBeschraenkungen.getInstance(0, 6, 2, 1, GridBagConstraints.HORIZONTAL, false));

		// Kontextmenü für Streitwerttabelle
		pmStreitwert = new JPopupMenu(STREITWERTE_POPUPLABEL);
		pmStreitwert.add(new JMenuItem(STREITWERTE_POPUPLABEL));
		pmStreitwert.addSeparator();
		pmStreitwert.add(new JMenuItem(druckeSwTabelleAction));
		pmStreitwert.add(new JMenuItem(kopiereSwTabelleAction));
		getTabStreitwertTabelle().addMouseListener(new KontextMouseListener(getTabStreitwertTabelle(), pmStreitwert));

		// Kontextmenü für gttm
		pmGttmRechner = new JPopupMenu(RECHNER_POPUPLABEL);
		pmGttmRechner.add(new JMenuItem(RECHNER_POPUPLABEL));
		pmGttmRechner.addSeparator();
		pmGttmRechner.add(new JMenuItem(neueGkgGebuehrAction));
		pmGttmRechner.add(new JMenuItem(neueRvgVerguetungAction));
		pmGttmRechner.add(new JMenuItem(saeumnisGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(anerkenntnisGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(normaleGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(mitInstanzGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(eintragEntfernenAktion));
		tabGebuehrenTatbestaende.addMouseListener(new KontextMouseListener(tabGebuehrenTatbestaende, pmGttmRechner));
	}

	/**
	 * Die Methode leert alle Tabellen und sonstige Oberflächenelemente. Damit kann
	 * man dann die Registerkarte initialisieren oder auf den Anfangszustand
	 * zurücksetzen.
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.WerteLesbar#leereWerte()
	 */
	@Override
	public void leereWerte() {
		setzeGebuehrenRechnerTabelle(0);
	}

	/**
	 * Die Methode leert die GebuehrenRechnerTabelle oder füllt sie mit
	 * üblicherweise anfallenden Gebühren, lässt aber die Listener in Ruhe.
	 * 
	 * @param key 0 = Tabelle leeren, {@link #GEBUEHREN_ANERKENNTNIS} = Tabelle mit
	 *            den bei einem Anerkenntnis üblicherweise anfallenden Gebühren
	 *            füllen, {@link #GEBUEHREN_SAEUMNIS} Tabelle mit den bei einer
	 *            Säumnis üblicherweise anfallenden Gebühren füllen
	 * 
	 */
	public void setzeGebuehrenRechnerTabelle(int key) {
		// GebuehrenTatbestand[] values;
		List<GebuehrenTatbestand> values;
		switch (key) {
		case GEBUEHREN_STANDARD:
			// values = new GebuehrenTatbestand[]
			values = new ArrayList<GebuehrenTatbestand>();
			values.addAll(Arrays.asList(new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1210,
					AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104,
					AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R }));
			break;
		case GEBUEHREN_ANERKENNTNIS:
			// values = new GebuehrenTatbestand[]
			values = new ArrayList<GebuehrenTatbestand>();
			values.addAll(Arrays.asList(new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1211,
					AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104,
					AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R }));
			break;
		case GEBUEHREN_SAEUMNIS:
			// values = new GebuehrenTatbestand[]
			values = new ArrayList<GebuehrenTatbestand>();
			values.addAll(Arrays.asList(new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1210,
					AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3105, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R }));
			break;
		case GEBUEHREN_2INSTANZ:
			// values = new GebuehrenTatbestand[]
			values = new ArrayList<GebuehrenTatbestand>();
			values.addAll(Arrays.asList(new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1210,
					GerichtsGebuehrenTabelle.KV1220, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104,
					AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV3200, AnwaltsGebuehrenTabelle.VV3104,
					AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R, AnwaltsGebuehrenTabelle.VV3100,
					AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV3200,
					AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R }));
			break;
		default:
			values = new ArrayList<GebuehrenTatbestand>();
			break;
		}
		if (gttmRechner == null) {
			gttmRechner = new GebuehrenTatbestandTableModel(values);
			gttmRechner.addTableModelListener(aktualisierungsHorcher);
			tabGebuehrenTatbestaende.setModel(gttmRechner);
		} else {
			gttmRechner.clear();
			gttmRechner.addAll(values);
		}
	}

	/**
	 * Die Methode leert die GebuehrenRechnerTabelle, füllt sie mit den Gebühren aus
	 * {@link eu.gronos.kostenrechner.controller.VerfahrensDatenContainer.GebuehrenBerechnung}
	 * und setzt auch den Streitwert entsprechend.
	 * 
	 * @param container ein
	 *                  {@link eu.gronos.kostenrechner.controller.VerfahrensDatenContainer.GebuehrenBerechnung}
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#setzeWerte(java.lang.Object)
	 */
	public void setzeWerte(final VerfahrensDatenContainer.GebuehrenBerechnung container) {
		/* Erst den Streitwert eintragen und die Tabelle springen lassen */
		getFtfStreitwert().setText(Long.toString((long) container.streitwert));
		aktualisierungsHorcher.focusLost(new FocusEvent(getFtfStreitwert(), FocusEvent.FOCUS_LOST));
		/* Dann die Tatbestände in die Gebührentabelle eintragen */
		if (gttmRechner == null) {
			gttmRechner = new GebuehrenTatbestandTableModel(container.gebuehren);
			gttmRechner.addTableModelListener(aktualisierungsHorcher);
			tabGebuehrenTatbestaende.setModel(gttmRechner);
		} else {
			gttmRechner.clear();
			gttmRechner.addAll(container.gebuehren);
		}
	}

	/**
	 * Die Methode dient dazu, eine Referenz auf das Hauptfenster zu übermitteln
	 * 
	 * @param kostenrechner
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#setKostenrechner(eu.gronos.kostenrechner.Kostenrechner)
	 */
	@Override
	public void setKostenrechner(Kostenrechner kostenrechner) {
		this.kostenrechner = kostenrechner;
	}

	/**
	 * Die Methode addiert die Gebühren der Tabelle.
	 * 
	 */
	public void errechneGebuehrenSumme() {
		// tfGebuehrenSumme.requestFocus();
		String streitwert = "";
		double gebuehrenSumme = 0.0;
		if (getFtfStreitwert() != null && gttmRechner != null) {
			try {
				streitwert = getFtfStreitwert().getText();
				gebuehrenSumme = gttmRechner.errechneGebuehrenSumme(Long.parseLong(streitwert));
			} catch (NumberFormatException e) {
				// e.printStackTrace();

				new FehlerDialog(FehlerDialog.FEHLER_BLANKO,
						"Falsches Eingabeformat für den Streitwert: \"" + streitwert + "\"!\n ", e).showDialog();
			}
		}
		String text = String.format(SUMME_EUR, gebuehrenSumme);
		if (tfGebuehrenSumme != null)
			tfGebuehrenSumme.setText(text);
	}

	/**
	 * Die Methode dient dazu, alle Programmzustände in einen
	 * {@link VerfahrensDatenContainer} zu packen
	 * 
	 * @return den Untercontainer von
	 *         {@link VerfahrensDatenContainer#gebuehrenBerechnung}
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#leseWerte()
	 */
	@Override
	public GebuehrenBerechnung leseWerte() {
		GebuehrenBerechnung container = new GebuehrenBerechnung();
		container.streitwert = 0L;
		container.gebuehren = new ArrayList<GebuehrenTatbestand>();
		try {
			container.streitwert = Long.parseLong(getFtfStreitwert().getText());
		} catch (NumberFormatException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
		}
		container.gebuehren = gttmRechner.getAllValues();// toArray();

		return container;
	}

	/**
	 * @return gibt {@link #tabStreitwertTabelle} als JTable zurück.
	 */
	public JTable getTabStreitwertTabelle() {
		return tabStreitwertTabelle;
	}

	/**
	 * @param tabStreitwertTabelle d. {@link #tabStreitwertTabelle}, d. gesetzt
	 *                             werden soll als JTable
	 */
	public void setTabStreitwertTabelle(JTable tabStreitwertTabelle) {
		this.tabStreitwertTabelle = tabStreitwertTabelle;
	}

	/**
	 * @return gibt {@link #ftfStreitwert} als JFormattedTextField zurück.
	 */
	public JFormattedTextField getFtfStreitwert() {
		return ftfStreitwert;
	}

	/**
	 * @param ftfStreitwert d. {@link #ftfStreitwert}, d. gesetzt werden soll als
	 *                      JFormattedTextField
	 */
	public void setFtfStreitwert(JFormattedTextField ftfStreitwert) {
		this.ftfStreitwert = ftfStreitwert;
	}

	/**
	 * @return gibt {@link #kostenrechner} als Kostenrechner zurück.
	 */
	public Kostenrechner getKostenrechner() {
		return kostenrechner;
	}

	@Override
	public GebuehrenTatbestandTableModel getTableModel(int index) {
		return gttmRechner;
	}

}
