/**
 * HilfePanel.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.helpanderror;

import java.awt.BorderLayout;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.StyledDocument;
import javax.swing.text.html.HTMLEditorKit;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.KostenJPanel;

/**
 * Registerkarte 4 Hilfe
 *
 * @author Peter Schuster (setrok)
 * @version
 * @date 22.06.2018
 *
 */
public class HilfePanel extends KostenJPanel {

	private static final long serialVersionUID = 6247368145017378263L;
	private static HilfePanel instance = new HilfePanel();
	@SuppressWarnings("unused")
	private Kostenrechner kostenrechner;
	
	/**
	 * Konstruktor ist private, weil Singleton. 
	 * 
	 * Besonderheit: BorderLayout statt GridBagLayout
	 * 
	 */
	private HilfePanel() {
		super();
		setLayout(new BorderLayout());
	}

	/**
	 * Die Methode setzt die Registerkarte 4 Hilfe mit einem einzigen
	 * JScrollPane im JPanel und darin einen JTextPane mit Hilfetext.
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#buildPanel()
	 */
	@Override
	public void buildPanel() {
		JTextPane textPane = new JTextPane();
		HTMLEditorKit html = new HTMLEditorKit();
		textPane.setEditorKit(html);
		textPane.setEditable(false);
		StyledDocument doc = (StyledDocument) html.createDefaultDocument();
		textPane.setStyledDocument(doc);
		leseHtml(doc, html);
		// Caret ganz nach oben setzen
		textPane.setCaretPosition(0);
		textPane.addHyperlinkListener(HinzufuegenDialog.HYPERLINK_LISTENER);
		add(new JScrollPane(textPane), BorderLayout.CENTER);
	}

	/**
	 * Die Methode dient dazu, TODO
	 * 
	 * @param kostenrechner
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#setKostenrechner(eu.gronos.kostenrechner.Kostenrechner)
	 */
	@Override
	public void setKostenrechner(Kostenrechner kostenrechner) {
		this.kostenrechner = kostenrechner;
	}

	/**
	 * @return gibt {@link #instance} als HilfePanel zurück.
	 */
	public static HilfePanel getInstance() {
		return instance;
	}
	
	/**
	 * Liest den Inhalt einer HTML-Datei ein und schreibt ihn formatiert in das
	 * übergebene StyledDocument
	 * 
	 * @param doc
	 *            das StyledDocument des JTextPanes, das gefüllt werden soll
	 * @param html
	 *            das HTMLEditorKit des JTextPanes, das gefüllt werden soll.
	 */
	private void leseHtml(StyledDocument doc, HTMLEditorKit html) {
		InputStream is;
		try {
			is = getClass().getClassLoader().getResourceAsStream("resources/hilfetext.html");
			doc.remove(0, doc.getLength());
			doc.putProperty("IgnoreCharsetDirective", new Boolean(true));
			html.read(is, doc, 0);
		} catch (FileNotFoundException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
			// Kostenrechner.zeigeFehler(e.getLocalizedMessage());
			// e.printStackTrace();
		} catch (IOException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
			// Kostenrechner.zeigeFehler(e.getLocalizedMessage());
			// e.printStackTrace();
		} catch (BadLocationException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
			// Kostenrechner.zeigeFehler(e.getLocalizedMessage());
			// e.printStackTrace();
		}
	}

}
