/**
 * TenorDialog.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.result;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.rtf.RTFEditorKit;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.KontextMouseListener;
import eu.gronos.kostenrechner.controller.TenorBegruendungAction;
import eu.gronos.kostenrechner.controller.TenorDatenContainer;
import eu.gronos.kostenrechner.controller.TenorierendAdapter;
import eu.gronos.kostenrechner.controller.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.controller.files.SchreibeEmailTenorFalschAction;
import eu.gronos.kostenrechner.controller.files.SpeichereRtfAction;
import eu.gronos.kostenrechner.controller.files.RtfZwischenablageAction;
import eu.gronos.kostenrechner.controller.files.RtfDocHelper;
import eu.gronos.kostenrechner.interfaces.Tabulierend;
import eu.gronos.kostenrechner.interfaces.Tenorierend;
import eu.gronos.kostenrechner.logic.TenorUtils;
import eu.gronos.kostenrechner.model.tenordaten.TabulierendTableModel;
import eu.gronos.kostenrechner.interfaces.Begruendend;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Ein JDialog, der einen Tenor anzeigt und diesen auf Wunsch als RTF speichert
 * und diesen auch in die Zwischenablage kopieren kann.
 * 
 * @author Peter Schuster (setrok)
 * @date 15.06.2014
 */
public class TenorDialog extends HinzufuegenDialog<Void> {

	private static final String POPUPLABEL_TENORTEXT = "Tenortext...";
	private static final long serialVersionUID = -7570808761464484864L;
	private JScrollPane scpKostentenor;
	JTextPane txtpnKostentenor;
	Tenorierend tenorierend;
	final VerfahrensDatenContainer container;
	private RtfDocHelper rtfHelper;
	private final FontUndFensterHelfer fufHelfer = new FontUndFensterHelfer(this);
	private final AbstractAction tenorInZwischenablage = new RtfZwischenablageAction(this,
			"Tenor in Zwischenablage kopieren", "Kopiert den gesamten Tenortext in die Zwischenablage.", KeyEvent.VK_K,
			"copy", KeyStroke.getKeyStroke(KeyEvent.VK_C, Kostenrechner.ctrlMask));
	private final AbstractAction tenorSpeichern = new SpeichereRtfAction(this, "Tenor speichern",
			"Speichert den Tenortext im Rich Text Format (RTF)", KeyEvent.VK_S, "save",
			KeyStroke.getKeyStroke(KeyEvent.VK_S, Kostenrechner.ctrlMask));
	private final AbstractAction begruendungEinfuegen = new TenorBegruendungAction(this, "Begründung hinzufügen",
			"Fügt die hinterlegte Begründung für den generierten Tenor an.");

	private final AbstractAction emailSchreibenTenorFalsch;

	/**
	 * bereitsGruendeDa speichert, ob die Begründung bereits eingefügt wurde, als
	 * boolean
	 */
	private boolean bereitsGruendeDa = false;

	/**
	 * Der Konstruktor verwendet statt eines {@link Tenorierend} jetzt einen
	 * {@link TenorDatenContainer}.
	 * 
	 * @param owner               der JFrame, der den JDialog aufruft
	 * @param tenorDatenContainer der {@link TenorDatenContainer} mit den Daten für
	 *                            den Tenor
	 * @param container           ein VerfahrensDatenContainer
	 */
	public TenorDialog(JFrame owner, TenorDatenContainer tenorDatenContainer, VerfahrensDatenContainer container) {
		super(owner, tenorDatenContainer.berechnungsTyp);
		this.tenorierend = new TenorierendAdapter(tenorDatenContainer);
		this.container = container;
		super.setOkButtonToBeFocussed(true);
		emailSchreibenTenorFalsch = new SchreibeEmailTenorFalschAction("Tenor falsch",
				"<html>Öffnet den Standard-E-Mail-Programm,<br>um eine E-Mail an den Programmierer zu verfassen</html>",
				KeyEvent.VK_E, "e-mail", this.tenorierend, this.container);
	}

	/**
	 * Die Methode holt das StyledDocument aus der JTextPane
	 * 
	 * @return the model
	 */
	public StyledDocument getDocument() {
		return txtpnKostentenor.getStyledDocument();
	}

	/**
	 * Die Methode dient dazu, an den Tenortext Gründe anzuhängen, falls es sich
	 * beim {@link #tenorierend} um ein {@link Begruendend} handelt und die Gründe
	 * nicht bereits angehängt wurden.
	 * 
	 */
	public void setBegruendungText() {
		if (tenorierend instanceof Begruendend && !bereitsGruendeDa) {
			final String gruende = ((Begruendend) tenorierend).getGruende();
			try {
				DefaultStyledDocument doc = (DefaultStyledDocument) getDocument();
				int caret = txtpnKostentenor.getCaretPosition();
				int anfang = doc.getLength();
				doc.insertString(anfang, TenorUtils.GRUENDE_UEBERSCHRIFT, doc.getStyle("bold"));
				int nachUeberschrift = doc.getLength();
				doc.insertString(nachUeberschrift, gruende, doc.getStyle("base"));
				if (istTenorierendTabulierend()) {
					gruendeTabelleEinpassen(doc, gruende, nachUeberschrift);
				}
				Kostenrechner.getLogger().info(
						String.format("caret: %04d; doc.Length()-anfang: %04d", caret, (doc.getLength() - anfang)));
				txtpnKostentenor.setCaretPosition(caret);
				bereitsGruendeDa = true;
			} catch (BadLocationException ble) {
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, ble.getLocalizedMessage(), ble);
			}
		}
	}

	/**
	 * @return gibt {@link #rtfHelper} als RtfDocHelper zurück.
	 */
	public RtfDocHelper getRtfHelper() {
		return rtfHelper;
	}

	/**
	 * @param rtfHelper d. {@link #rtfHelper}, d. gesetzt werden soll als
	 *                  RtfDocHelper
	 */
	public void setRtfHelper(RtfDocHelper rtfHelper) {
		this.rtfHelper = rtfHelper;
	}

	/**
	 * Die Methode fuellt das ContentPane mit einer {@link JTextPane} für den
	 * Tenortext des {@link Begruendend} in einer {@link JScrollPane}. Zudem setzt
	 * sie ein paar mehr {@link JButton Schaltflächen}
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#fuelleContentPane()
	 */
	@Override
	protected void fuelleContentPane() {
		txtpnKostentenor = new JTextPane();
		txtpnKostentenor.setEditorKit(new RTFEditorKit());
		txtpnKostentenor.setEditable(false);
		setRtfHelper(new RtfDocHelper((RTFEditorKit) txtpnKostentenor.getEditorKit(), getDocument()));
		scpKostentenor = new JScrollPane(txtpnKostentenor);
		contentPanel.add(scpKostentenor,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 3, 1, GridBagConstraints.BOTH, true));
		getButtonPane().add(new JButton(tenorInZwischenablage));
		addToRootInputMap(tenorInZwischenablage);

		getButtonPane().add(new JButton(tenorSpeichern));
		addToRootInputMap(tenorSpeichern);

		getButtonPane().add(new JButton(emailSchreibenTenorFalsch));
		addToRootInputMap(emailSchreibenTenorFalsch);

		setKostentenorText();
		// txtpnKostentenor.setContentType("text/rtf; charset=8859_1");
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

		/* Kontextmenü zusammensetzen */
		JPopupMenu pmTenortext = new JPopupMenu(POPUPLABEL_TENORTEXT);
		pmTenortext.add(new JMenuItem(POPUPLABEL_TENORTEXT));
		pmTenortext.addSeparator();
		pmTenortext.add(new JMenuItem(tenorSpeichern));
		pmTenortext.add(new JMenuItem(tenorInZwischenablage));
		pmTenortext.add(new JMenuItem(begruendungEinfuegen));
		/*
		 * Das Einfügen von Begründungen geht nur bei Klassen, die UrteilsTenorierend
		 * implementieren; sonst fehlt die entsprechende Methode
		 */
		if (tenorierend instanceof Begruendend)
			begruendungEinfuegen.setEnabled(true);
		else
			begruendungEinfuegen.setEnabled(false);
		txtpnKostentenor.addMouseListener(new KontextMouseListener(pmTenortext));
	}

	/**
	 * Die Methode setKostentenorText dient dazu, den Tenor der übergegebenen
	 * Tenorklasse im JTextPane mit der passenden Schriftart und Formatierungen zu
	 * setzen. Als Dialogtitel wird die Tenorbeschreibung des Tenorierend gesetzt.
	 * 
	 * @param tenorierend die Tenor erzeugende Klasseninstanz, die "Tenorierend"
	 *                    implementiert.
	 * 
	 * @see javax.swing.text.Document#insertString(int,String,AttributeSet)
	 */
	void setKostentenorText() {
		final String tenor = tenorierend.erzeugeTenor();
		// Beschreibung vom "Tenorierend" holen
		try {
			DefaultStyledDocument doc = (DefaultStyledDocument) getDocument();
			Style baseStyle = fufHelfer.createBaseStyle(doc);
			fufHelfer.createBoldStyle(doc, baseStyle);
			fufHelfer.createTabStoppedStyle(doc, baseStyle);

			if (istTenorierendTabulierend()) {
				createTableStyle(doc, baseStyle);
			}
			doc.remove(0, doc.getLength());
			doc.insertString(0, tenor, doc.getStyle("base"));
			fufHelfer.setParagraphAttributes(txtpnKostentenor, doc, 0, doc.getLength(),
					!(tenorierend instanceof Begruendend));
			// Die Beschreibung auch beim Speichern als Titel setzen
			doc.putProperty(Document.TitleProperty, tenorierend.getTenorBeschreibung());
			fufHelfer.setPreferredSizeFromAttributeSet(doc.getStyle("base"), FontUndFensterHelfer.MAX_SPALTEN,
					FontUndFensterHelfer.MAX_ZEILEN);
		} catch (BadLocationException ble) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, ble.getLocalizedMessage(), ble);
		}
	}

	/**
	 * Die Methode dient dazu, eine Begründungstabelle einzufügen, wenn möglich
	 * 
	 * @param doc
	 * @param gruende
	 * @param nachUeberschrift
	 * 
	 * @throws BadLocationException
	 * 
	 * @FIXED: Folgendes sorgte für eine NullPointerExeption beim
	 *         Speichern/Kopieren: setParagraphAttributes(doc, anfang,
	 *         doc.getLength() - anfang, false); Wohl wegen doc.getLenght-anfang
	 * 
	 *         {@url http://stackoverflow.com/questions/2600960/nullpointerexception-in-javax-swing-text-simpleattributeset-addattribute}
	 */
	private void gruendeTabelleEinpassen(DefaultStyledDocument doc, final String gruende, int nachUeberschrift)
			throws BadLocationException {
		// Wenn eine Begründungstabelle erzeugt werden kann, dann diese einfügen.
		Tabulierend tab = (Tabulierend) tenorierend;
		int[] range = tab.getRange();
		int anfangGruende = nachUeberschrift + range[0] + 1;
		int laenge = range[1] - range[0];
		// Damit es keine Dopplungen gibt, erst einmal den alten Text herausnehmen
		doc.remove(anfangGruende, laenge - 1);
		// Dann den neuen Text mit der richtigen Formatierung (d.h. mit JTable in
		// JScrollpane) einfügen
		doc.insertString(anfangGruende, gruende.substring(range[0] + 1, range[1]), doc.getStyle("table"));
		Kostenrechner.getLogger()
				.info(String.format("anfangGruende: %04d, doc.getLength(): %04d, range: (%04d, %04d); laenge: %04d",
						anfangGruende, doc.getLength(), range[0], range[1], laenge));
	}

	/**
	 * Die Methode definiert einen Style mit einer JTable ("table"), sofern das
	 * übergebene {@link Tenorierend} auch das Interface {@link Tabulierend}
	 * implementiert
	 * 
	 * @param doc
	 * @param baseStyle
	 * @return
	 * @url "http://www.java-forums.org/awt-swing/2260-how-insert-tables-into-jtextpane.html#post4296"
	 */
	private Style createTableStyle(StyledDocument doc, Style baseStyle) {
		Style style = doc.addStyle("table", baseStyle);
		JTable table = baueGruendeTabelle();
		JScrollPane jScrollPane = new JScrollPane(table);
		StyleConstants.setComponent(style, jScrollPane);
		return style;
	}/* , Style style */

	/**
	 * Die Methode dient dazu, mit den Tabellendaten eines {@link Tabulierend} eine
	 * JTable zu bauen und mit den passenden Renderern zu versehen.
	 * 
	 * @return die gebaute JTable
	 */
	private JTable baueGruendeTabelle() {
		JTable table = new JTable(new TabulierendTableModel((Tabulierend) tenorierend));
		// Dimension d = table.getMinimumSize();
		Dimension d = table.getPreferredSize();
		d.width = txtpnKostentenor.getPreferredSize().width;
		table.setPreferredScrollableViewportSize(d);
		table.setDefaultRenderer(Double.class, new NachkommastellenRenderer());
		table.setDefaultRenderer(String.class, new StringButtonRenderer());
		return table;
	}

	/**
	 * @return true, sofern das übergebene {@link Tenorierend} auch das Interface
	 *         {@link Tabulierend} implementiert
	 */
	private boolean istTenorierendTabulierend() {
		return tenorierend instanceof Tabulierend;
	}

	/**
	 * Die Methode dient der Kompatibilität zur Oberklasse und tut nichts.
	 * 
	 * @return null
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#baueRueckgabewert()
	 */
	@Override
	protected Void baueRueckgabewert() {
		return null;
	}

	/**
	 * Die Methode dient der Kompatibilität zur Oberklasse und liefert immer true
	 * 
	 * @return true
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#parseEingabe()
	 */
	@Override
	protected boolean parseEingabe() {
		return true;
	}

}
